@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -40,6 +40,8 @@
 #include "libxml/xpath.h"
 #include "libfolia/folia.h"
 
+using namespace icu;
+
 namespace folia {
   void initMT();
 
@@ -79,29 +81,33 @@ namespace folia {
   class Document {
     friend bool operator==( const Document&, const Document& );
     friend std::ostream& operator<<( std::ostream&, const Document * );
+    enum Mode { NOMODE=0, PERMISSIVE=1, CHECKTEXT=2, FIXTEXT=4, STRIP=8 };
+    friend class Processor;
   public:
     Document();
-    Document( const std::string& );
+    explicit Document( const KWargs& );
+    explicit Document( const std::string& s): Document( getArgs(s) ){};
     ~Document();
     void init();
     bool readFromFile( const std::string& );
     bool readFromString( const std::string& );
-    bool save( std::ostream&, const std::string&, bool = false );
-    bool save( std::ostream& os, bool kanon = false ){
+    bool save( std::ostream&, const std::string&, bool = false ) const;
+    bool save( std::ostream& os, bool kanon = false ) const {
       return save( os, "", kanon );
     }
-    bool save( const std::string&, const std::string&, bool = false );
-    bool save( const std::string& s, bool kanon = false ){
+    bool save( const std::string&, const std::string&, bool = false ) const ;
+    bool save( const std::string& s, bool kanon = false ) const {
       return save( s, "", kanon );
     }
+    std::string xmlstring( bool k = false ) const;
     int size() const;
     FoliaElement* doc() const { return foliadoc; }
     Text* addText( const KWargs& );
     Text* addText( Text * );
-    FoliaElement* append( Text *t ){
-      // almost backward compatible
-      return addText(t);
-    };
+    Speech* addSpeech( const KWargs& );
+    Speech* addSpeech( Speech * );
+    FoliaElement* append( FoliaElement *t ); // OBSOLETE
+    FoliaElement* setRoot( FoliaElement * );
     void set_foreign_metadata( xmlNode * );
     void addStyle( const std::string&, const std::string& );
     void replaceStyle( const std::string&, const std::string& );
@@ -127,8 +133,8 @@ namespace folia {
     bool toXml( const std::string&,
 		const std::string& ="",
 		bool = false ) const;
-    std::string metadatatype() const { return _metadatatype; };
-    std::string metadatafile() const { return _metadatafile; };
+    std::string metadatatype() const;
+    std::string metadatafile() const;
     void set_metadata( const std::string& type, const std::string& value );
     const std::string get_metadata( const std::string& type ) const;
 
@@ -144,6 +150,10 @@ namespace folia {
 		     const std::string&,
 		     const std::string& );
     bool isDeclared( AnnotationType::AnnotationType, const std::string& = "" );
+    std::string unalias( AnnotationType::AnnotationType,
+			 const std::string& ) const;
+    std::string alias( AnnotationType::AnnotationType,
+		       const std::string& ) const;
     std::string defaultset( AnnotationType::AnnotationType ) const;
 
     std::string defaultannotator( AnnotationType::AnnotationType,
@@ -157,20 +167,30 @@ namespace folia {
     FoliaElement* parseXml( );
 
     std::string id() const { return _id; };
-    std::string language() const { return _language; };
+    std::string language() const;
     void declare( AnnotationType::AnnotationType,
 		  const std::string&,
 		  const std::string& = "" );
     void declare( AnnotationType::AnnotationType,
 		  const std::string&, const std::string&,
-		  const std::string&, const std::string& );
+		  const std::string&, const std::string&,
+		  const std::string& = "" );
+    void un_declare( AnnotationType::AnnotationType,
+		     const std::string& );
     xmlDoc *XmlDoc() const { return _xmldoc; };
     xmlNs *foliaNs() const { return _foliaNsOut; };
     void keepForDeletion( FoliaElement *p ) { delSet.insert( p ); };
     void addExternal( External *p ) { externals.push_back( p ); };
-    FoliaElement *resolveExternals( FoliaElement* );
+    void resolveExternals();
     int debug;
-    bool permissive() const { return mode == "permissive"; };
+    bool permissive() const { return mode & PERMISSIVE; };
+    bool checktext() const {
+      return mode & CHECKTEXT;
+    };
+    bool fixtext() const {
+      return mode & FIXTEXT;
+    };
+    bool strip() const { return mode & STRIP; };
     class at_t {
       friend std::ostream& operator<<( std::ostream&, const at_t& );
     public:
@@ -179,17 +199,49 @@ namespace folia {
       std::string t;
       std::string d;
     };
+    void incrRef( AnnotationType::AnnotationType, const std::string& );
+    void decrRef( AnnotationType::AnnotationType, const std::string& );
+    void setmode( const std::string& ) const;
+    std::string getmode() const;
+    std::multimap<AnnotationType::AnnotationType,std::string> unused_declarations( ) const;
+      const MetaData *get_submetadata( const std::string& m ){
+      const auto& it = submetadata.find( m );
+      if ( it == submetadata.end() ){
+	return 0;
+      } else {
+	return it->second;
+      }
+    }
+    void cache_textcontent( TextContent *tc ){
+      t_offset_validation_buffer.push_back( tc );
+    }
+    void cache_phoncontent( PhonContent *tc ){
+      p_offset_validation_buffer.push_back( tc );
+    }
+    bool validate_offsets() const;
+    int compare_to_lib_version() const;
+    std::string version() const { return _version_string; };
+    std::string update_version();
+    bool version_below( int, int );
+    std::map<AnnotationType::AnnotationType,std::multimap<std::string,at_t> > annotationdefaults() const { return _annotationdefaults; };
+    void parse_metadata( const xmlNode * );
+    void setDocumentProps( KWargs& );
   private:
-    std::map<AnnotationType::AnnotationType,std::multimap<std::string,at_t> > annotationdefaults;
-    std::vector<std::pair<AnnotationType::AnnotationType,std::string>> anno_sort;
-    FoliaElement* parseFoliaDoc( xmlNode * );
-    void parsemeta( xmlNode * );
+    std::map<AnnotationType::AnnotationType,std::multimap<std::string,at_t> > _annotationdefaults;
+    std::vector<std::pair<AnnotationType::AnnotationType,std::string>> _anno_sort;
+    std::map<AnnotationType::AnnotationType,std::map<std::string,int> > _annotationrefs;
+    std::map<AnnotationType::AnnotationType,std::map<std::string,std::string>> _alias_set;
+    std::map<AnnotationType::AnnotationType,std::map<std::string,std::string>> _set_alias;
+    std::vector<TextContent*> t_offset_validation_buffer;
+    std::vector<PhonContent*> p_offset_validation_buffer;
+
     void setimdi( xmlNode * );
-    void setDocumentProps( KWargs&  );
-    void parseannotations( xmlNode * );
+    void parseannotations( const xmlNode * );
+    void parsesubmeta( const xmlNode * );
     void getstyles();
     void setannotations( xmlNode *) const;
     void setmetadata( xmlNode * ) const;
+    void addsubmetadata( xmlNode *) const;
     void setstyles( xmlDoc* ) const;
     xmlDoc *to_xmlDoc( const std::string& ="", bool=false ) const;
     std::map<std::string, FoliaElement* > sindex;
@@ -203,20 +255,16 @@ namespace folia {
     const xmlChar* _foliaNsIn_href;
     const xmlChar* _foliaNsIn_prefix;
     mutable xmlNs *_foliaNsOut;
-    std::string _metadatatype;
-    xmlNode *_metadata;
-    std::vector<ForeignData *> _foreigndata;
-    std::string _metadatafile;
-    std::string _title;
-    std::string _date;
-    std::string _language;
-    std::string _publisher;
-    std::string _license;
-    std::map<std::string,std::string> meta_atts;
+    MetaData *_metadata;
+    std::map<std::string,MetaData *> submetadata;
     std::multimap<std::string,std::string> styles;
-    std::string mode;
-    std::string filename;
-    std::string version;
+    mutable Mode mode;
+    std::string _source_filename;
+    std::string _version_string;
+    int major_version;
+    int minor_version;
+    int sub_version;
+    std::string patch_version;
     bool external;
     Document( const Document& ); // inhibit copies
     Document& operator=( const Document& ); // inhibit copies
@@ -238,6 +286,12 @@ namespace folia {
     return os;
   }
 
+  void expand_version_string( const std::string&,
+			      int&, int&, int&, std::string& );
+
+  std::string library_version();
+  std::string folia_version();
+
 } // namespace folia
 
 #endif // FOLIA_DOCUMENT_H
