@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -50,62 +50,75 @@ namespace folia {
 
   class ArgsError: public std::runtime_error {
   public:
-  ArgsError( const std::string& s ): std::runtime_error( "error in argument list: " + s  ){};
+    explicit ArgsError( const std::string& s ): std::runtime_error( "error in argument list: " + s  ){};
   };
 
   class KeyError: public std::out_of_range {
   public:
-  KeyError(): std::out_of_range( "key out of range" ){};
+    explicit KeyError(): std::out_of_range( "key out of range" ){};
+    KeyError( const std::string& s ): std::out_of_range( "key out of range: " + s ){};
   };
 
   class NotImplementedError: public std::runtime_error {
   public:
-  NotImplementedError( const std::string& s ):
+    explicit NotImplementedError( const std::string& s ):
     std::runtime_error( "NOT IMPLEMENTED: " + s ){};
   };
 
   class ValueError: public std::runtime_error {
   public:
-  ValueError( const std::string& s ): std::runtime_error( s ){};
+    explicit ValueError( const std::string& s ): std::runtime_error( s ){};
   };
 
   class XmlError: public std::runtime_error {
   public:
-  XmlError( const std::string& s ): std::runtime_error( "XML error: " + s ){};
+    explicit XmlError( const std::string& s ): std::runtime_error( "XML error: " + s ){};
+  };
+
+  class MetaDataError: public std::runtime_error {
+  public:
+    explicit MetaDataError( const std::string& s ): std::runtime_error( "MetaData problem: " + s ){};
   };
 
   class NoSuchAnnotation: public std::runtime_error {
   public:
-  NoSuchAnnotation( const std::string& s ): std::runtime_error( "no such annotation: " + s ){};
+    explicit NoSuchAnnotation( const std::string& s ): std::runtime_error( "no such annotation: " + s ){};
   };
 
   class NoSuchText: public std::runtime_error {
   public:
-  NoSuchText( const std::string& s ): std::runtime_error( "no such text: " + s ){};
+    explicit NoSuchText( const std::string& s ): std::runtime_error( "no such text: " + s ){};
   };
 
   class NoSuchPhon: public std::runtime_error {
   public:
-  NoSuchPhon( const std::string& s ): std::runtime_error( "no such phoneme: " + s ){};
+    explicit NoSuchPhon( const std::string& s ): std::runtime_error( "no such phoneme: " + s ){};
   };
 
   class DuplicateAnnotationError: public std::runtime_error {
   public:
-  DuplicateAnnotationError( const std::string& s ): std::runtime_error( s ){};
+    explicit DuplicateAnnotationError( const std::string& s ): std::runtime_error( s ){};
   };
 
   class DuplicateIDError: public std::runtime_error {
   public:
-  DuplicateIDError( const std::string& s ): std::runtime_error( "duplicate ID : " + s ){};
+    explicit DuplicateIDError( const std::string& s ): std::runtime_error( "duplicate ID : " + s ){};
   };
 
   class NoDefaultError: public std::runtime_error {
   public:
-  NoDefaultError( const std::string& s ): std::runtime_error( "No Default found: " + s ){};
+    explicit NoDefaultError( const std::string& s ): std::runtime_error( "No Default found: " + s ){};
   };
 
-  UnicodeString UTF8ToUnicode( const std::string& );
-  std::string UnicodeToUTF8( const UnicodeString& );
+  class InconsistentText: public std::runtime_error {
+  public:
+    explicit InconsistentText( const std::string& s ): std::runtime_error( "inconsistent text: " + s ){};
+  };
+
+  class UnresolvableTextContent: public std::runtime_error {
+  public:
+    explicit UnresolvableTextContent( const std::string& s ): std::runtime_error( "Unresolvable text: " + s ){};
+  };
 
   std::string toString( const AnnotationType::AnnotationType& );
   AnnotationType::AnnotationType stringToAT( const std::string& );
@@ -120,6 +133,12 @@ namespace folia {
   KWargs getArgs( const std::string& );
   std::string toString( const KWargs& );
 
+  inline std::ostream& operator<<( std::ostream& os,
+				   const KWargs& ats ){
+    os << toString( ats );
+    return os;
+  }
+
   inline std::ostream& operator<<( std::ostream& os,
 				   const AnnotationType::AnnotationType& at ){
     os << toString( at );
@@ -138,6 +157,8 @@ namespace folia {
 
   bool isNCName( const std::string& );
 
+  icu::UnicodeString normalize_spaces( const icu::UnicodeString& );
+
 } // namespace folia
 
 namespace TiCC {
