@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -36,24 +36,17 @@
 #include "ticcutils/StringOps.h"
 #include "ticcutils/XMLtools.h"
 #include "ticcutils/PrettyPrint.h"
+#include "ticcutils/Unicode.h"
 #include "libfolia/folia.h"
 #include "libfolia/folia_properties.h"
 
 using namespace std;
+using namespace icu;
 using namespace TiCC;
+using namespace icu;
 
 namespace folia {
 
-  UnicodeString UTF8ToUnicode( const string& s ){
-    return UnicodeString::fromUTF8( s );
-  }
-
-  string UnicodeToUTF8( const UnicodeString& s ){
-    string result;
-    s.toUTF8String(result);
-    return result;
-  }
-
   FoliaElement *FoliaImpl::createElement( const string& tag,
 					  Document *doc ){
 
@@ -61,7 +54,7 @@ namespace folia {
     try {
       et = stringToET( tag );
     }
-    catch ( ValueError& e ){
+    catch ( const ValueError& e ){
       cerr << e.what() << endl;
       return 0;
     }
@@ -86,7 +79,6 @@ namespace folia {
     bool quoted = false;
     bool parseatt = true;
     bool escaped = false;
-    vector<string> parts;
     string att;
     string val;
     for ( const auto& let : s ){
@@ -236,12 +228,11 @@ namespace folia {
   }
 
   int toMonth( const string& ms ){
-    int result = 0;
     try {
-      result = stringTo<int>( ms );
+      int result = stringTo<int>( ms );
       return result - 1;
     }
-    catch( exception ){
+    catch( const exception& ){
       string m = TiCC::lowercase( ms );
       if ( m == "jan" )
 	return 0;
@@ -287,7 +278,7 @@ namespace folia {
       }
     }
     //    cerr << "found " << num << " parts" << endl;
-    tm *time = new tm();
+    tm time = tm();
     if ( num == 1 || num == 2 ){
       //      cerr << "parse date " << date_time[0] << endl;
       vector<string> date_parts;
@@ -295,15 +286,17 @@ namespace folia {
       switch ( dnum ){
       case 3: {
 	int mday = stringTo<int>( date_parts[2] );
-	time->tm_mday = mday;
+	time.tm_mday = mday;
       }
+	// fall through
       case 2: {
 	int mon = toMonth( date_parts[1] );
-	time->tm_mon = mon;
+	time.tm_mon = mon;
       }
+	// fall through
       case 1: {
 	int year = stringTo<int>( date_parts[0] );
-	time->tm_year = year-1900;
+	time.tm_year = year-1900;
       }
 	break;
       default:
@@ -321,15 +314,17 @@ namespace folia {
 	// ignore
       case 3: {
 	int sec = stringTo<int>( date_parts[2] );
-	time->tm_sec = sec;
+	time.tm_sec = sec;
       }
+	// fall through
       case 2: {
 	int min = stringTo<int>( date_parts[1] );
-	time->tm_min = min;
+	time.tm_min = min;
       }
+	// fall through
       case 1: {
 	int hour = stringTo<int>( date_parts[0] );
-	time->tm_hour = hour;
+	time.tm_hour = hour;
       }
 	break;
       default:
@@ -339,8 +334,7 @@ namespace folia {
     }
     // cerr << "read _date time = " << toString(time) << endl;
     char buf[100];
-    strftime( buf, 100, "%Y-%m-%dT%X", time );
-    delete time;
+    strftime( buf, 100, "%Y-%m-%dT%X", &time );
     return buf;
   }
 
@@ -350,25 +344,23 @@ namespace folia {
     }
     //    cerr << "try to read a time " << s << endl;
     vector<string> time_parts;
-    string mil = "000";
-    tm *time = new tm();
+    tm time = tm();
     int num = TiCC::split_at( s, time_parts, ":" );
     if ( num != 3 ){
       cerr << "failed to read a time " << s << endl;
       return "";
     }
-    time->tm_min = stringTo<int>( time_parts[1] );
-    time->tm_hour = stringTo<int>( time_parts[0] );
+    time.tm_min = stringTo<int>( time_parts[1] );
+    time.tm_hour = stringTo<int>( time_parts[0] );
     string secs = time_parts[2];
     num = TiCC::split_at( secs, time_parts, "." );
-    time->tm_sec = stringTo<int>( time_parts[0] );
+    time.tm_sec = stringTo<int>( time_parts[0] );
     string mil_sec = "000";
     if ( num == 2 ){
       mil_sec = time_parts[1];
     }
     char buf[100];
-    strftime( buf, 100, "%X", time );
-    delete time;
+    strftime( buf, 100, "%X", &time );
     string result = buf;
     result += "." + mil_sec;
     //    cerr << "formatted time = " << result << endl;
@@ -395,7 +387,7 @@ namespace folia {
 	try {
 	  stringTo<AnnotationType::AnnotationType>( s );
 	}
-	catch ( ValueError& e ){
+	catch ( const ValueError& e ){
 	  cerr << "no AnnotationType found for string '" << s << "'" << endl;
 	  sane = false;
 	}
@@ -432,7 +424,7 @@ namespace folia {
 	try {
 	  et2 = stringToET( s );
 	}
-	catch ( ValueError& e ){
+	catch ( const ValueError& e ){
 	  cerr << "no element type found for string '" << s << "'" << endl;
 	  sane = false;
 	  continue;
@@ -447,7 +439,7 @@ namespace folia {
 	try {
 	  tmp1 = FoliaImpl::createElement( s );
 	}
-	catch( ValueError &e ){
+	catch( const ValueError &e ){
 	  string err = e.what();
 	  if ( err.find("abstract") ==string::npos ){
 	    cerr << "createElement(" << s << ") failed! :" << err << endl;
@@ -459,7 +451,7 @@ namespace folia {
 	  try {
 	    tmp2 = FoliaImpl::createElement( et );
 	  }
-	  catch( ValueError &e ){
+	  catch( const ValueError &e ){
 	    string err = e.what();
 	    if ( err.find("abstract") == string::npos ){
 	      cerr << "createElement(" << int(et) << ") failed! :" << err << endl;
@@ -492,4 +484,26 @@ namespace folia {
     return true;
   }
 
+  UnicodeString normalize_spaces( const UnicodeString& input ){
+    // substitute \n \r \t by spaces AND all multiple spaces by 1
+    // also trims at back and front.
+    UnicodeString result;
+    bool is_space = false;
+    for ( int i=0; i < input.length(); ++i ){
+      if ( u_isspace( input[i] ) ){
+	if ( is_space ){
+	  continue;
+	}
+	is_space = true;
+	result += " ";
+      }
+      else {
+	is_space = false;
+	result += input[i];
+      }
+    }
+    result.trim(); // remove leading and trailing whitespace;
+    return result;
+  }
+
 } //namespace folia
